import { addImportsSources, addComponent, useNuxt, addPlugin, defineNuxtModule, createResolver } from '@nuxt/kit';

const libraryName = "vue-map-ui";
const runtimeDir = "./runtime";
const allComponents = [
  "VMapAttributionControl",
  "VMapControlPosition",
  "VMapLayersControl",
  "VMapLocateControl",
  "VMapScaleControl",
  "VMapZoomControl",
  "VMapDefaultIcon",
  "VMapDivIcon",
  "VMapDivMarker",
  "VMapIcon",
  "VMapIconMarker",
  "VMapMarker",
  "VMapPinIcon",
  "VMapPinMarker",
  "VMapTileLayer",
  "VMap",
  "VMapPane",
  "VMapArcGisAeroTileLayer",
  "VMapArcGisTileLayer",
  "VMapGoogleTileLayer",
  "VMapMapboxTileLayer",
  "VMapOpenTransportTileLayer",
  "VMapOsmTileLayer"
];
const allImports = [
  "useMap",
  "usePane",
  "useTileLayer",
  "useMarker",
  "useIcon",
  "useDivIcon",
  "useAttributionControl",
  "useLayersControl",
  "useScaleControl",
  "useZoomControl",
  "useControlPosition",
  "useLocateControl"
];
const defaults = {
  components: allComponents,
  imports: allImports,
  autoImports: true,
  styles: true,
  extensions: false
};

function resolveImports(config) {
  const { imports } = config;
  const allImports = new Set(imports);
  addImportsSources({
    from: libraryName,
    imports: [...allImports]
  });
}

function isArray(value) {
  return Array.isArray(value);
}
function toArray(value) {
  return isArray(value) ? value : [value];
}
function isObject(value) {
  return Object.prototype.toString.call(value) === "[object Object]";
}

function resolveComponents(config, resolver) {
  const { components } = config;
  const allComponents = new Set(components);
  for (const item of allComponents) {
    const [name, alias] = toArray(item);
    addComponent({
      export: name,
      name: alias || name,
      filePath: libraryName,
      mode: "client"
    });
    addComponent({
      name: alias || name,
      filePath: resolver.resolve(
        runtimeDir,
        "components",
        "ServerPlaceholder.vue"
      ),
      mode: "server"
    });
  }
}

function resolveStyles(config) {
  const nuxt = useNuxt();
  const enabledCss = isArray(nuxt.options.css);
  if (!enabledCss) {
    return;
  }
  const { styles } = config;
  const allStyles = styles === true;
  const isObj = isObject(styles);
  if (allStyles || isObj && styles.leafletStyle) {
    nuxt.options.css.push("leaflet/dist/leaflet.css");
  }
  if (allStyles || isObj && styles.normalizeStyle) {
    nuxt.options.css.push(`${libraryName}/dist/normalize.css`);
  }
  if (allStyles || isObj && styles.libraryStyle) {
    nuxt.options.css.push(`${libraryName}/dist/style.css`);
  }
  if (allStyles || isObj && styles.themeStyle) {
    nuxt.options.css.push(`${libraryName}/dist/theme-all.css`);
  }
}

const name = "nuxt-vue-map-ui";
const version = "1.0.12";

function resolveExtensions(config, resolver) {
  const nuxt = useNuxt();
  const { extensions } = config;
  const allEnabled = extensions === true;
  const isObj = isObject(extensions);
  if (allEnabled || isObj && extensions.locateControl) {
    nuxt.options.css.push(
      "@leaflet-extensions/locatecontrol/dist/leaflet-locatecontrol.css"
    );
    addPlugin({
      src: resolver.resolve(
        runtimeDir,
        "plugins",
        "locateControlExtension.client.ts"
      )
    });
  }
}

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "vueMap"
  },
  defaults,
  setup(options, nuxt) {
    const resolver = createResolver(import.meta.url);
    resolveStyles(options);
    nuxt.options.imports.autoImport !== false && options.autoImports && resolveImports(options);
    nuxt.options.components !== false && resolveComponents(options, resolver);
    resolveExtensions(options, resolver);
  }
});

export { module as default };
